"""Things that are needed for convenient parallel Execution"""

from PyFoam.Basics.Utilities import Utilities
from os import path

class LAMMachine(Utilities):
    """Wrapper class for starting an stopping a LAM-Machine"""
    def __init__(self,machines):
        """@param machines: Name of the file with the machine information"""

        Utilities.__init__(self)

        self.stop()
        self.mFile=machines
        self.boot()

    def stop(self):
        """Stops a LAM-machine (if one is running)"""
        self.running=False
        self.execute("lamhalt -v")
        
    def boot(self):
        """Boots a LAM-machine using the machine-file"""
        self.execute("lamboot -s -v "+self.mFile)
        self.running=True

    def cpuNr(self):
        if self.running:
            lines=self.execute("lamnodes")
            nr=0
            for l in lines:
                tmp=l.split(':')
                nr+=int(tmp[1])
            return nr
        else:
            return -1

    def writeMetis(self,sDir):
        """Write the parameter-File for a metis decomposition
        @param sDir: Solution directory
        @type sDir: PyFoam.RunDictionary.SolutionDirectory"""

        params="method metis;\n"

        self.writeDecomposition(sDir,params)

    def writeSimple(self,sDir,direction):
        """Write the parameter-File for a metis decomposition
        @param sDir: Solution directory
        @type sDir: PyFoam.RunDictionary.SolutionDirectory
        @param direction: direction in which to decompose (0=x, 1=y, 2=z)"""

        params ="method simple;\n"
        params+="\nsimpleCoeffs\n{\n\t n \t ("
        if direction==0:
            params+=str(self.cpuNr())+" "
        else:
            params+="1 "
        if direction==1:
            params+=str(self.cpuNr())+" "
        else:
            params+="1 "
        if direction==2:
            params+=str(self.cpuNr())
        else:
            params+="1"
        params+=");\n\t delta \t 0.001;\n}\n"
        
        self.writeDecomposition(sDir,params)

    def writeDecomposition(self,sDir,par):
        """Write parameter file for a decomposition
        @param par:Parameters specific for that kind of decomposition
        @type par:str
        @param sDir: Solution directory
        @type sDir: PyFoam.RunDictionary.SolutionDirectory"""

        f=open(path.join(sDir.systemDir(),"decomposeParDict"),"w")
        self.writeDictionaryHeader(f)
        f.write("// * * * * * * * * * //\n\n")
        f.write("numberOfSubdomains "+str(self.cpuNr())+";\n\n")
        f.write(par)
        f.write("\n\n// * * * * * * * * * //")
        f.close()
