"""Check for Execution-Time information"""

import re

from FileLineAnalyzer import FileLineAnalyzer

class ExecutionTimeLineAnalyzer(FileLineAnalyzer):
    """Parses lines for the execution time"""

    executionRegexp="^ExecutionTime = (.+) s$"
    
    def __init__(self):
        FileLineAnalyzer.__init__(self,titles=["ExecutionTime"])
        self.exp=re.compile(self.executionRegexp)
        self.lastTime=0.
        
    def doAnalysis(self,line):
        """Writes total execution time and time needed since last
        time-step"""
        m=self.exp.match(line)
        if m!=None:
            time=float(m.group(1))
            
            self.files.write("executionTime",self.parent.getTime(),(time,time-self.lastTime))

            self.lastTime = time
    
