"""Base class for analyzing lines"""

class LogLineAnalyzer:
    """Base class for the analysis of all lines from a OpenFOAM-log

    Lines are available one at a time"""
    
    def __init__(self):
        self.parent=None

    def doAnalysis(self,line):
        """Analyze a line

        line - the line to be analyzed

        This method carries the main functionality in the sub-classes"""
        pass

    def setParent(self,parent):
        """Introduces the LineAnalyzer to its supervisor

        parent - The Analyzer class of which this is a part"""
        self.parent=parent

    def setDirectory(self,oDir):
        """Set the directory to which output is to be written (if any
        output is written)"""
        pass
    
    def goOn(self):
        """If the analyzer thinks the simulation should be stopped
        (for instance because of convergence) it returns false"""
        return True

    def getTime(self):
        """@returns: current time"""
        return self.parent.getTime()
