#! /usr/bin/python

import sys,string,ConfigParser

from os import path,uname
from time import time,localtime,asctime
from PyFoam.Execution.BasicRunner import BasicRunner
from PyFoam.LogAnalysis.BoundingLogAnalyzer import BoundingLogAnalyzer
from PyFoam.FoamInformation import foamTutorials
from PyFoam.RunDictionary.SolutionDirectory import SolutionDirectory
from PyFoam.RunDictionary.SolutionFile import SolutionFile
from PyFoam.RunDictionary.ParameterFile import ParameterFile
from PyFoam.RunDictionary.BlockMesh import BlockMesh
from PyFoam.Execution.ParallelExecution import LAMMachine
from PyFoam.Basics.Utilities import Utilities

if len(sys.argv)<2:
    print "No config file specified"
    sys.exit(-1)

config=ConfigParser.ConfigParser()
files=sys.argv[1:]
good=config.read(files)
# will work with 2.4
# if len(good)!=len(files):
#    print "Problem while trying to parse files",files
#    print "Only ",good," could be parsed"
#    sys.exit(-1)

benchName=config.get("General","name")
isParallel=config.getboolean("General","parallel")
lam=None

if isParallel:
    nrCpus=config.getint("General","nProcs")
    machineFile=config.get("General","machines")
    lam=LAMMachine(machineFile)
    if lam.cpuNr()!=nrCpus:
        print "Wrong number of CPUs: ",lam.cpuNr()
        sys.exit(-1)

    print "Running parallel on",lam.cpuNr(),"CPUs"
    
benchCases=[]
config.remove_section("General")

for sec in config.sections():
    print "Reading: ",sec
    sol=config.get(sec,"solver")
    cas=config.get(sec,"case")
    pre=eval(config.get(sec,"prepare"))
    con=eval(config.get(sec,"controlDict"))
    bas=config.getfloat(sec,"baseline")
    wei=config.getfloat(sec,"weight")
    add=[]
    if config.has_option(sec,"additional"):
        add=eval(config.get(sec,"additional"))
        print "Adding: ", add
    util=[]
    if config.has_option(sec,"utilities"):
        util=eval(config.get(sec,"utilities"))
        print "Utilities: ", util    
    nr=99999
    if config.has_option(sec,"nr"):
        nr=eval(config.get(sec,"nr"))
    sp=None
    if config.has_option(sec,"blockSplit"):
        sp=eval(config.get(sec,"blockSplit"))
    toRm=[]
    if config.has_option(sec,"filesToRemove"):
        toRm=eval(config.get(sec,"filesToRemove"))
    setInit=[]
    if config.has_option(sec,"setInitial"):
        setInit=eval(config.get(sec,"setInitial"))

    parallelOK=False
    if config.has_option(sec,"parallelOK"):
        parallelOK=config.getboolean(sec,"parallelOK")

    deMet=["metis"]
    if config.has_option(sec,"decomposition"):
        deMet=config.get(sec,"decomposition").split()

    if deMet[0]=="metis":
        pass
    elif deMet[0]=="simple":
        if len(deMet)<2:
            deMet.append(0)
        else:
            deMet[1]=int(deMet[1])
    else:
        print "Unimplemented decomposition method",deMet[0],"switching to metis"
        deMet=["metis"]
        
    if isParallel==False or parallelOK==True:
        benchCases.append( (nr,sec,sol,cas,pre,con,bas,wei,add,util,sp,toRm,setInit,deMet) )
    else:
        print "Skipping",sec,"because not parallel"

benchCases.sort()

parallelString=""
if isParallel:
    parallelString=".cpus="+str(nrCpus)
    
resultFile=open("Benchmark."+benchName+"."+uname()[1]+parallelString+".results","w")

totalSpeedup=0
totalWeight =0
currentEstimate = 1.

print "\nStart Benching\n"

for nr,description,solver,case,prepare,control,base,weight,additional,utilities,split,toRemove,setInit,decomposition in benchCases:
    #    control.append( ("endTime",-2000) )
    print "Running Benchmark: ",description
    print "Solver: ",solver
    print "Case: ",case
    caseName=solver+"_"+case+"_"+benchName+"."+uname()[1]+".case"
    print "Short name: ",caseName
    caseDir=caseName+".runDir"
    
    tutDir=foamTutorials()
    workDir=path.realpath(path.curdir)

    orig=SolutionDirectory(path.join(tutDir,solver,case),archive=None)
    for a in additional+utilities:
        orig.addToClone(a)
    orig.cloneCase(path.join(workDir,caseDir))

    run=BasicRunner(silent=True,argv=[solver,workDir,caseDir],logname="BenchRunning",lam=lam)
    runDir=run.getSolutionDirectory()
    controlFile=ParameterFile(runDir.controlDict())

    for rm in toRemove:
        fn=path.join(caseDir,rm)
        print "Removing file",fn
        runner=Utilities()
        runner.execute("rm -f "+fn)
        
    for field,bc,val in setInit:
        print "Setting",field,"on",bc,"to",val
        SolutionFile(runDir.initialDir(),field).replaceBoundary(bc,val)
        
    oldDeltaT=controlFile.replaceParameter("deltaT",0)
    
    for u in utilities:
        print "Building utility ",u
        runner=Utilities()
        runner.execute("wmake 2>&1 >%s %s" % (path.join(caseDir,"BenchCompile."+u),path.join(caseDir,u)))
        
    print "Preparing the case: "
    if lam!=None:
        prepare=prepare+[("decomposePar","")]
        if decomposition[0]=="metis":
            lam.writeMetis(SolutionDirectory(path.join(workDir,caseDir)))
        elif decomposition[0]=="simple":
            lam.writeSimple(SolutionDirectory(path.join(workDir,caseDir)),decomposition[1])

    if split:
        print "Splitting the mesh:",split
        bm=BlockMesh(runDir.blockMesh())
        bm.refineMesh(split)
        
    for pre,post in prepare:
        print "Doing ",pre," ...."
        args=string.split("%s %s %s %s" % (pre,workDir,caseDir,post))
        util=BasicRunner(silent=True,argv=args,logname="BenchPrepare_"+pre)
        util.start()

    controlFile.replaceParameter("deltaT",oldDeltaT)
    
    #    control.append(("endTime",-1000))
    for name,value in control:
        controlFile.replaceParameter(name,value)
        
    print "Starting at ",asctime(localtime(time()))
    print " Baseline is %f, estimated speedup %f -> estimated end at %s " % (base,currentEstimate,asctime(localtime(time()+base/currentEstimate)))
    print "Running the case ...."
    run.start()

    speedup=0
    cpuUsage=0
    try:
        speedup=base/run.run.wallTime()
        cpuUsage=100.*run.run.cpuTime()/run.run.wallTime()
    except ZeroDivisionError:
        print "Division by Zero: ",run.run.wallTime()
        
    totalSpeedup+=speedup*weight
    totalWeight +=weight
    
    print "Wall clock: ",run.run.wallTime()
    print "Speedup: ",speedup," (Baseline: ",base,")"
    print "CPU Time: ",run.run.cpuTime()
    print "CPU Time User: ",run.run.cpuUserTime()
    print "CPU Time System: ",run.run.cpuSystemTime()
    print "Memory: ",run.run.usedMemory()
    print "CPU Usage: %6.2f%%" % (cpuUsage)
    print
    print
    
    resultFile.write("Case %s WallTime %g CPUTime %g UserTime %g SystemTime %g Memory %d Speedup %g\n" %(caseName,run.run.wallTime(),run.run.cpuTime(),run.run.cpuUserTime(),run.run.cpuSystemTime(),run.run.usedMemory(),speedup))

    resultFile.flush()

    currentEstimate=totalSpeedup/totalWeight
    
if lam!=None:
    lam.stop()
    
print "Total Speedup: ",currentEstimate," ( ",totalSpeedup," / ",totalWeight, " )"

resultFile.write("Total Speedup: %g\n" % (currentEstimate))

resultFile.close()
